using System;
using OpenNETCF.IO;
using OpenNETCF.Threading;

namespace AppliedData.IO.CAN
{	
	public delegate void CanEventDelegate();

	/// <summary>
	/// Class for managing the Controller Area Network (CAN) Bus implemented on the Philips SJA1000
	/// </summary>
	public class SJA1000 : StreamInterfaceDriver
	{
		/// <summary>
		/// Fired when a CAN message is received
		/// </summary>
		public event CanEventDelegate MessageReceived;
		/// <summary>
		/// Fired when an error is detected on the CAN bus
		/// </summary>
		public event CanEventDelegate OnError;
		/// <summary>
		/// Fired when a buffer overrun is detected on the CAN bus
		/// </summary>
		public event CanEventDelegate OnOverrun;

		private BusNumber m_busNumer = BusNumber.CAN1;
		private bool m_dataReady = false;

		// todo: add properties so user can alter these if they alter the registry
		private string m_dataReadyEventPrefix = "EV_DATA_RDY";
		private string m_appEventPrefix = "EV_APP";
		private string m_errorEventPrefix = "EV_ERR";
		private string m_overrunEventPrefix = "EV_OVRERR";

		// threads are cheap - we'll have one dedicated to each event type
		private ThreadEx m_dataReadyMonitorThread;
		private ThreadEx m_appEventMonitorThread;
		private ThreadEx m_errorMonitorThread;
		private ThreadEx m_overrunMonitorThread;

		private System.Windows.Forms.Control m_invokerControl = new System.Windows.Forms.Control();
		private EventType m_eventType;

		private bool m_killThreads = false;

		[Flags]
		private enum EventType
		{
			Receive = 1,
			Error = 2,
			Overrun = 4
		}

		/// <summary>
		/// Creates an instance of the Bus class
		/// </summary>
		/// <param name="busNumber">The bus number to connect to</param>
		public SJA1000(BusNumber busNumber)
		{
			m_busNumer = busNumber;

			if(busNumber == BusNumber.CAN1)
			{
				PortName = "CAN1:";
			}
			else
			{
				PortName = "CAN2:";
			}

			// start up the event threads

			m_dataReadyMonitorThread = new ThreadEx(new System.Threading.ThreadStart(DataReadyEventThreadProc));
			m_appEventMonitorThread = new ThreadEx(new System.Threading.ThreadStart(AppEventThreadProc));
			m_errorMonitorThread = new ThreadEx(new System.Threading.ThreadStart(ErrorEventThreadProc));
			m_overrunMonitorThread = new ThreadEx(new System.Threading.ThreadStart(OverrunEventThreadProc));

			m_dataReadyMonitorThread.Start();
			m_appEventMonitorThread.Start();
			m_errorMonitorThread.Start();
			m_overrunMonitorThread.Start();
		}

		~SJA1000()
		{
			// signal all threads to shut down
			m_killThreads = true;
		}

		#region --- Event Monitoring Threads ---
		private void EventMarshalProc(object o, EventArgs a)
		{
			lock(m_invokerControl)
			{
				if((m_eventType & EventType.Receive) == EventType.Receive)
				{
					// clear the event bit
					m_eventType &= ~EventType.Receive;

					// raise the event on the caller's thread
					if(MessageReceived != null)
					{
						MessageReceived();
					}
				}

				if((m_eventType & EventType.Error) == EventType.Error)
				{
					// clear the event bit
					m_eventType &= ~EventType.Error;

					// raise the event on the caller's thread
					if(OnError != null)
					{
						OnError();
					}
				}

				if((m_eventType & EventType.Overrun) == EventType.Overrun)
				{
					// clear the event bit
					m_eventType &= ~EventType.Overrun;

					// raise the event on the caller's thread
					if(OnOverrun != null)
					{
						OnOverrun();
					}
				}
			}
		}

		private void DataReadyEventThreadProc()
		{
			string eventName = string.Format("{0}{1}", m_dataReadyEventPrefix, (int)m_busNumer);

			// create a wait handle
			EventWaitHandle waitHandle = new EventWaitHandle(
				false,
				EventResetMode.AutoReset,
				eventName);

			while(!m_killThreads)
			{
				// this event will be set when data is ready, unset when not
				m_dataReady = waitHandle.WaitOne(1000, true);
			}
		}

		private void AppEventThreadProc()
		{
			string eventName = string.Format("{0}{1}", m_appEventPrefix, (int)m_busNumer);

			// create a wait handle
			EventWaitHandle waitHandle = new EventWaitHandle(
				false,
				EventResetMode.AutoReset,
				eventName);

			while(!m_killThreads)
			{
				// keep checking back every second to see if the thread has been killed
				if(waitHandle.WaitOne(1000, true))
				{
					m_eventType |= EventType.Receive;
					m_invokerControl.Invoke(new EventHandler(EventMarshalProc));
				}
			}
		}

		private void ErrorEventThreadProc()
		{
			string eventName = string.Format("{0}{1}", m_errorEventPrefix, (int)m_busNumer);

			// create a wait handle
			EventWaitHandle waitHandle = new EventWaitHandle(
				false,
				EventResetMode.AutoReset,
				eventName);

			while(!m_killThreads)
			{
				// keep checking back every second to see if the thread has been killed
				if(waitHandle.WaitOne(1000, true))
				{
					m_eventType |= EventType.Error;
					m_invokerControl.Invoke(new EventHandler(EventMarshalProc));
				}
			}
		}

		private void OverrunEventThreadProc()
		{
			string eventName = string.Format("{0}{1}", m_overrunEventPrefix, (int)m_busNumer);

			// create a wait handle
			EventWaitHandle waitHandle = new EventWaitHandle(
				false,
				EventResetMode.AutoReset,
				eventName);

			while(!m_killThreads)
			{
				// keep checking back every second to see if the thread has been killed
				if(waitHandle.WaitOne(1000, true))
				{
					m_eventType |= EventType.Overrun;
					m_invokerControl.Invoke(new EventHandler(EventMarshalProc));
				}
			}
		}

		#endregion


		/// <summary>
		/// Resets the controller chip
		/// </summary>
		public void ResetController()
		{
			base.DeviceIoControl((int)IOCTL.ResetChip, null, null);
		}

		/// <summary>
		/// Returns <b>true</b> if bus data is currently avaialble
		/// </summary>
		public bool DataAvailable
		{
			get { return m_dataReady; }
		}

		/// <summary>
		/// Clears all data from the Receive queue
		/// </summary>
		public void ClearQueue()
		{
			base.DeviceIoControl((int)IOCTL.ClearQueue, null, null);
		}

		/// <summary>
		/// Sets or Gets the controller's baud rate
		/// </summary>
		public int BaudRate
		{
			set 
			{ 
				base.DeviceIoControl((int)IOCTL.SetBaudRate, BitConverter.GetBytes(value), null); 
			}
			get
			{
				byte[] data = new byte[4];
				base.DeviceIoControl((int)IOCTL.GetBaudRate, null, data);
				return BitConverter.ToInt32(data, 0);
			}
		}

		/// <summary>
		/// Sets or gets the controller's Sample Point
		/// </summary>
		public int SamplePoint
		{
			set 
			{ 
				base.DeviceIoControl((int)IOCTL.SetSamplePoint, BitConverter.GetBytes(value), null); 
			}
			get
			{
				byte[] data = new byte[4];
				base.DeviceIoControl((int)IOCTL.GetSamplePoint, null, data);
				return BitConverter.ToInt32(data, 0);
			}
		}

		/// <summary>
		/// Returns the number of messages currently in the Receive queue
		/// </summary>
		public int MessageCount
		{
			get
			{
				byte[] data = new byte[4];
				base.DeviceIoControl((int)IOCTL.GetNumMsgs, null, data);
				return BitConverter.ToInt32(data, 0);
			}
		}

		/// <summary>
		/// Returns the current state of the controller's status register
		/// </summary>
		public uint StatusRegister
		{
			get
			{
				byte[] data = new byte[4];
				base.DeviceIoControl((int)IOCTL.GetStatusRegister, null, data);
				return BitConverter.ToUInt32(data, 0);
			}
		}

		/// <summary>
		/// Sends a Message on the bus
		/// </summary>
		/// <param name="message"></param>
		public void SendMessage(Message message)
		{
			base.Write(message);
		}

		/// <summary>
		/// Gets the next available message from teh Receive queue
		/// </summary>
		/// <returns></returns>
		public Message GetMessage()
		{
			return Read(Message.MESSAGE_LENGTH);
		}
	}
}
