using System;
using System.Data;
using OpenNETCF.IO;

namespace AppliedData.IO
{
	/// <remarks>
	/// Used to access non-SmartIO Digital I/O on ADS systems
	/// </remarks>
	public sealed class DigitalIO : StreamInterfaceDriver
	{
		private void WriteInt(int data)
		{
			this.Write(BitConverter.GetBytes(data), 4);
		}

		private int ReadInt()
		{
			return BitConverter.ToInt32(this.Read(4), 0);
		}

		private int IOCTLRead(IOCTL ioctl)
		{
			byte[] data = new byte[4];

			DeviceIoControl((int)ioctl, null, data);

			return BitConverter.ToInt32(data, 0);
		}

		private void IOCTLWrite(IOCTL ioctl, int value)
		{
			byte[] data = BitConverter.GetBytes(value);

			DeviceIoControl((int)ioctl, data, null);
		}

        private void IOCTLWrite(IOCTL ioctl, int value1, int value2)
		{
            byte[] data = new byte[8];
			(BitConverter.GetBytes(value1)).CopyTo(data, 0);
            (BitConverter.GetBytes(value2)).CopyTo(data, 4);
            
			DeviceIoControl((int)ioctl, data, null);
		}

		public void Open()
		{
			base.Open(FileAccess.All, FileShare.ReadWrite);
		}

		public void Close()
		{
			base.Close();
		}

		public DigitalIO() : base("DIO1:")
		{
		}
		
        public int StateBits
		{
			get 
			{ 
				return ReadInt(); 
			}
			set 
			{ 
				WriteInt(value); 
			}
		}

        /*
        public String DriverVersion
        {
            get
            {
                byte[] version = new byte[10];
                DeviceIoControl((int)IOCTL.DIO_IOCTL_GET_DRIVER_VERSION, null, version);
                return version.ToString();
            }
        }
        */

		public int InputCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_INPUTS); }
		}

		public int OutputCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_OUTPUTS); }
		}

		public int InterruptCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_INT); }
		}

		public int WakeCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_WAKEUP_INT); }
		}

		public int DirectionMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_GET_DIRECTION); }
			set { IOCTLWrite(IOCTL.DIO_IOCTL_SET_DIRECTION, value) ; }
		}

        /// <summary>
        /// Enables the passed interrupt
        /// </summary>
        /// <param name="interrupt">The interrupt to be enabled</param>
        /// <returns>True when interrupt enabled, false when interrupt fails to enable because the DIO pin is not interrupt capable</returns>

        public bool EnableInterrupt(Interrupt interrupt) 
        {
            int trigger = 0;

            // Test against interrupt mask
            if ((InterruptCapableMask & interrupt.mask) == 0)
                return false;
            else
            {
                if (interrupt.risingTrigger)
                    trigger |= (int)IOCTL.DIO_TRIGGER_RISING;
                if (interrupt.fallingTrigger)
                    trigger |= (int)IOCTL.DIO_TRIGGER_FALLING;

                IOCTLWrite(IOCTL.DIO_IOCTL_ENABLE_INT, interrupt.mask, trigger);

                return true;
            }
        }

        /// <summary>
        /// Disable the passed interrupt
        /// </summary>
        /// <param name="interrupt">The interrupt to be disabled</param>
        public void DisableInterrupt(Interrupt interrupt)
        {
            IOCTLWrite(IOCTL.DIO_IOCTL_DISABLE_INT, interrupt.mask);    
        }

        public int GetInterruptTriggerMode
        {
            get { return IOCTLRead(IOCTL.DIO_IOCTL_GET_INT); }
        }

        [Flags]
		private enum IOCTL
		{
			/*
			DIO_IOCTL_INPUT 				= 0x00,
			DIO_IOCTL_OUTPUT 				= 0x01,
			DIO_IOCTL_HIGH					= 0x02,
			DIO_IOCTL_LOW					= 0x03,
			DIO_IOCTL_ERR_LINE_MASKED		= 0x04,
			*/

			DIO_IOCTL_CAPABLE_INPUTS		= 0x100,
			DIO_IOCTL_CAPABLE_OUTPUTS		= 0x101,
			DIO_IOCTL_GET_DRIVER_MASK		= 0x102,
			DIO_IOCTL_SET_DRIVER_MASK		= 0x103,
			DIO_IOCTL_GET_DIRECTION			= 0x104,
			DIO_IOCTL_SET_DIRECTION			= 0x105,

			DIO_IOCTL_CAPABLE_INT			= 0x110,
			DIO_IOCTL_CAPABLE_WAKEUP_INT	= 0x111,
			DIO_IOCTL_GET_INT				= 0x112,
			DIO_IOCTL_ENABLE_INT			= 0x113,
			DIO_IOCTL_DISABLE_INT			= 0x114,
            
            DIO_IOCTL_SET_INT_THREAD_PRIORITY = 0x115,
			//DIO_IOCTL_GET_DRIVER_VERSION	= 0xA0,
            
            DIO_TRIGGER_RISING				=	0x01,
            DIO_TRIGGER_FALLING				=	0x02,
            DIO_TRIGGER_WAKEUP_RISING		=	0x04,
            DIO_TRIGGER_WAKEUP_FALLING		=	0x08,
            DIO_TRIGGER_DISABLED			=	0x00
		}
	}

    /// <summary>
    /// Used to define an interrupt and is passed to DigitalIO.EnableInterrupt() and DigitalIO.DisableInterrupt()
    /// </summary>
    public class Interrupt
    {
        /// <summary>
        /// Interrupt Constructor
        /// </summary>
        /// <param name="number">The Interrupt Number</param>
        /// <param name="risingTrigger">true for a rising edge trigger, false otherwise</param>
        /// <param name="fallingTrigger">true for a falling edge trigger, false otherwise</param>
        public Interrupt(int number, bool risingTrigger, bool fallingTrigger)
        {
            this.number = number;
            this.risingTrigger = risingTrigger;
            this.fallingTrigger = fallingTrigger;

            // Create the bit mask
            this.mask = 1 << number;
        }

        public int number;
        public bool risingTrigger;
        public bool fallingTrigger;
        
        /// <summary>
        /// The bit mask associated with the interrupt number used by the DIO driver
        /// </summary>
        public int mask;
    }
}
