using System;
using System.Data;
using OpenNETCF.IO;

namespace AppliedData.IO
{
	/// <remarks>
	/// Used to access non-SmartIO Digital I/O on ADS systems
	/// </remarks>
	public sealed class DigitalIO : StreamInterfaceDriver
	{
		private void WriteInt(int data)
		{
			this.Write(BitConverter.GetBytes(data), 4);
		}

		private int ReadInt()
		{
			return BitConverter.ToInt32(this.Read(4), 0);
		}

		private int IOCTLRead(IOCTL ioctl)
		{
			byte[] data = new byte[4];

			DeviceIoControl((int)ioctl, null, data);

			return BitConverter.ToInt32(data, 0);
		}

		private void IOCTLWrite(IOCTL ioctl, int value)
		{
			byte[] data = BitConverter.GetBytes(value);

			DeviceIoControl((int)ioctl, data, null);
		}

        private void IOCTLWrite(IOCTL ioctl, int value1, int value2)
		{
            byte[] data = new byte[8];
			(BitConverter.GetBytes(value1)).CopyTo(data, 0);
            (BitConverter.GetBytes(value2)).CopyTo(data, 4);
            
			DeviceIoControl((int)ioctl, data, null);
		}

        /// <summary>
        /// Opens the DIO driver
        /// Once open, no other program can use the DIO driver till it is closed
        /// </summary>
		public void Open()
		{
			base.Open(FileAccess.All, FileShare.ReadWrite);
		}

        /// <summary>
        /// DigitaIO Constructor
        /// </summary>
		public DigitalIO() : base("DIO1:")
		{
		}
		
        /// <summary>
        /// Gets or sets the current state of the bits in the DIO driver
        /// </summary>
        public int StateBits
		{
			get 
			{ 
				return ReadInt(); 
			}
			set 
			{ 
				WriteInt(value); 
			}
		}

        /// <summary>
        /// Returns a mask containing 1's for all bits that are currently high
        /// </summary>
        public int HighBits
        {
            get { return IOCTLRead(IOCTL.DIO_IOCTL_HIGH);  }
        }

        /// <summary>
        /// Returns a mask containing 1's for all bits that are currently low
        /// </summary>
        public int LowBits
        {
            get { return IOCTLRead(IOCTL.DIO_IOCTL_LOW); }
        }
        
        /// <summary>
        /// Returns a string listing the current DIO driver version
        /// </summary>
        public String DriverVersion
        {
            get
            {
                byte[] dummy = new byte[1];
                byte[] version = new byte[10];
                DeviceIoControl((int)IOCTL.DIO_IOCTL_GET_DRIVER_VERSION, dummy, version);
                return version.ToString();
            }
        }
        
        /// <summary>
        /// Returns a mask containing 1's for all bits that are input capable
        /// </summary>
		public int InputCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_INPUTS); }
		}

        /// <summary>
        /// Returns a mask containing 1's for all bits that are output capable
        /// </summary>
		public int OutputCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_OUTPUTS); }
		}

        /// <summary>
        /// Returns a mask containing 1's for all bits that are interrupt capable
        /// </summary>
		public int InterruptCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_INT); }
		}

        /// <summary>
        /// Returns a mask containing 1's for all bits that are interrupt wakeup capable
        /// </summary>
		public int WakeCapableMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_CAPABLE_WAKEUP_INT); }
		}

        /// <summary>
        /// Gets or sets the direction for the bits in the DIO Driver.
        /// A 1 sets a bit as output.
        /// A 0 sets a bit as input.
        /// </summary>
		public int DirectionMask
		{
			get { return IOCTLRead(IOCTL.DIO_IOCTL_GET_DIRECTION); }
			set { IOCTLWrite(IOCTL.DIO_IOCTL_SET_DIRECTION, value) ; }
		}

        /// <summary>
        /// Gets or sets the driver mask for masking off bits in the DIO driver.
        /// A 1 sets a bit as usable.
        /// A 0 masks off the bit.
        /// </summary>
        public int DriverMask
        {
            get { return IOCTLRead(IOCTL.DIO_IOCTL_GET_DRIVER_MASK); }
            set { IOCTLWrite(IOCTL.DIO_IOCTL_SET_DRIVER_MASK, value); }
        }

        /// <summary>
        /// Sets the Thread Priority of the interrupt handling thread in the DIO driver.
        /// The default value is 240.
        /// </summary>
        public int ThreadPriority
        {
            set { IOCTLWrite(IOCTL.DIO_IOCTL_SET_INT_THREAD_PRIORITY, value); }
        }

        /// <summary>
        /// Enables the passed interrupt
        /// </summary>
        /// <param name="interrupt">The interrupt to be enabled</param>
        /// <returns>True when interrupt enabled, false when interrupt fails to enable because the DIO pin is not interrupt capable</returns>
        public bool EnableInterrupt(Interrupt interrupt) 
        {
            int trigger = 0;

            // Test against interrupt mask
            if ((InterruptCapableMask & interrupt.mask) == 0)
                return false;
            else
            {
                if (interrupt.risingTrigger)
                    trigger |= (int)TRIGGER.RISING;
                if (interrupt.fallingTrigger)
                    trigger |= (int)TRIGGER.FALLING;

                IOCTLWrite(IOCTL.DIO_IOCTL_ENABLE_INT, interrupt.mask, trigger);

                return true;
            }
        }

        /// <summary>
        /// Disable the passed interrupt
        /// </summary>
        /// <param name="interrupt">The interrupt to be disabled</param>
        public void DisableInterrupt(Interrupt interrupt)
        {
            IOCTLWrite(IOCTL.DIO_IOCTL_DISABLE_INT, interrupt.mask);    
        }

        /// <summary>
        /// Get the current trigger settings for the passed interrupt.
        /// Will set the risingTrigger and fallingTrigger members of the interrupt
        /// to the appropriate values based on the interrupt's current state 
        /// </summary>
        /// <param name="interrupt">The interrupt to obtain and get the current trigger settings</param>
        public void GetInterruptTrigger(ref Interrupt interrupt)
        {
            byte[] trigger = new byte[4];
            byte[] number = new byte[4];
            number = BitConverter.GetBytes(interrupt.mask);
            
            DeviceIoControl((int)IOCTL.DIO_IOCTL_GET_INT, number, trigger);
            int trig = Convert.ToInt32(trigger);

            interrupt.risingTrigger = false;
            interrupt.fallingTrigger = false;

            if ((trig & (int)TRIGGER.RISING) != 0)
                interrupt.risingTrigger = true;

            if ((trig & (int)TRIGGER.FALLING) != 0)
                interrupt.fallingTrigger = true;
        }

        /// <summary>
        /// IOCTL codes for the DIO driver
        /// </summary>
        [Flags]
		private enum IOCTL
		{
			DIO_IOCTL_INPUT 				= 0x00,
			DIO_IOCTL_OUTPUT 				= 0x01,
			DIO_IOCTL_HIGH					= 0x02,
			DIO_IOCTL_LOW					= 0x03,
			DIO_IOCTL_ERR_LINE_MASKED		= 0x04,
			
			DIO_IOCTL_CAPABLE_INPUTS		= 0x100,
			DIO_IOCTL_CAPABLE_OUTPUTS		= 0x101,
			DIO_IOCTL_GET_DRIVER_MASK		= 0x102,
			DIO_IOCTL_SET_DRIVER_MASK		= 0x103,
			DIO_IOCTL_GET_DIRECTION			= 0x104,
			DIO_IOCTL_SET_DIRECTION			= 0x105,

			DIO_IOCTL_CAPABLE_INT			= 0x110,
			DIO_IOCTL_CAPABLE_WAKEUP_INT	= 0x111,
			DIO_IOCTL_GET_INT				= 0x112,
			DIO_IOCTL_ENABLE_INT			= 0x113,
			DIO_IOCTL_DISABLE_INT			= 0x114,
            
            DIO_IOCTL_SET_INT_THREAD_PRIORITY = 0x115,
			DIO_IOCTL_GET_DRIVER_VERSION	= 0xA0,  
		}

        /// <summary>
        /// Values for indicating the type of trigger to set an interrupt
        /// </summary>
        [Flags]
        private enum TRIGGER
        {
            RISING				=	0x01,
            FALLING				=	0x02,
            WAKEUP_RISING		=	0x04,
            WAKEUP_FALLING		=	0x08,
            DISABLED			=	0x00
        }
	}

    /// <summary>
    /// Used to define an interrupt and is passed to DigitalIO.EnableInterrupt() and DigitalIO.DisableInterrupt()
    /// </summary>
    public class Interrupt
    {
        /// <summary>
        /// Interrupt Constructor
        /// </summary>
        /// <param name="number">The Interrupt Number</param>
        /// <param name="risingTrigger">true for a rising edge trigger, false otherwise</param>
        /// <param name="fallingTrigger">true for a falling edge trigger, false otherwise</param>
        public Interrupt(int number, bool risingTrigger, bool fallingTrigger)
        {
            this.number = number;
            this.risingTrigger = risingTrigger;
            this.fallingTrigger = fallingTrigger;

            // Create the bit mask
            this.mask = 1 << number;
        }

        /// <summary>
        /// Interrupt Constructor
        /// risingTrigger and fallingTrigger are set to false by default
        /// </summary>
        /// <param name="number">The Interrupt Number</param>
        public Interrupt(int number)
        {
            this.number = number;
            this.risingTrigger = false;
            this.fallingTrigger = false;

            // Create the bit mask
            this.mask = 1 << number;
        }
        
        /// <summary>
        /// The DIO number of the interrupt
        /// </summary>
        public int number;

        /// <summary>
        /// Flag indicating if the interrupt is set by a rising trigger
        /// </summary>
        public bool risingTrigger;

        /// <summary>
        /// Flag indicating if the interrupt is set by a falling trigger
        /// </summary>
        public bool fallingTrigger;
        
        /// <summary>
        /// The bit mask associated with the interrupt number used by the DIO driver
        /// </summary>
        public int mask;
    }
}
