using System;
using OpenNETCF.IO;
using OpenNETCF.Win32;
using System.Runtime.InteropServices;

namespace AppliedData.IO
{
    /// <remarks>
    /// Used to access the ADS Flash Driver
    /// </remarks>
    public sealed class Flash : StreamInterfaceDriver
    {
        /// <summary>
        /// Constructor to create a new Flash Driver object
        /// </summary>
        public Flash()
            : base("FSH1:")
        {
        }

        /// <summary>
        /// Opens the Flash driver for read and write access
        /// </summary>
        public void Open()
        {
            base.Open(FileAccess.All, FileShare.ReadWrite);
        }

        /// <summary>
        /// The Flash Chip Serial Number
        /// </summary>
        public long serialNumber
        {
            get
            {
                byte[] buffer = null;
                
                buffer = IOCTLRead(IOCTL.IOCTL_READ_FLASH_SER_NUM);

                return BitConverter.ToInt64(buffer, 0);
            }
        }

        /// <summary>
        /// The Flash Chip ID
        /// </summary>
        public long ID
        {
            get
            {
                byte[] buffer = null;

                buffer = IOCTLRead(IOCTL.IOCTL_READ_FLASH_ID);

                return BitConverter.ToInt64(buffer, 0);
            }
        }


        /// <summary>
        /// The Flash Chip Device ID
        /// </summary>
        public int deviceID
        {
            get
            {
                byte[] buffer = null;

                buffer = IOCTLRead(IOCTL.IOCTL_READ_FLASH_DEVICE_ID);

                // Error Check
                return BitConverter.ToInt32(buffer, 0);
            }
        }

        /// <summary>
        /// Stop the Flash Driver
        /// </summary>
        public void StopFlash()
        {
            IOCTLWrite(IOCTL.IOCTL_STOP_FLASHFX_DISK, 0);
        }

        /// <summary>
        /// Resume the Flash Driver
        /// </summary>
        public void ResumeFlash()
        {
            IOCTLWrite(IOCTL.IOCTL_RESUME_FLASHFX_DISK, 0);
        }


        /// <summary>
        /// Erase the Flash Disk
        /// </summary>
        public void EraseFlash()
        {
            IOCTLWrite(IOCTL.IOCTL_ERASE_FLASHFX_DISK, 0);
        }

        /// <summary>
        /// Test the driver with an invalid IOCTL
        /// </summary>
        public void InvalidIOCTLTest()
        {
            IOCTLWrite(IOCTL.IOCTL_INVALID_TEST, 0);
        }

        /// <summary>
        /// Read the IOCTL value from the Flash Driver
        /// </summary>
        /// <param name="ioctl">IOCTL to read</param>
        /// <returns>Current driver value of the IOCTL, null if error</returns>
        private byte[] IOCTLRead(IOCTL ioctl)
        {
            byte[] data = new byte[8];
            byte[] dummy = new byte[1];
            DeviceIoControl((int)ioctl, dummy, data);

            return data;
        }

        /// <summary>
        /// Write the value to the IOCTL in the Flash Driver
        /// </summary>
        /// <param name="ioctl">IOCTL to write</param>
        /// <param name="value">Value to write to the driver IOCTL</param>
        private void IOCTLWrite(IOCTL ioctl, int value)
        {
            byte[] data = BitConverter.GetBytes(value);

            DeviceIoControl((int)ioctl, data, null);
        }
        
        /// <summary>
        /// IOCTLs currently defined for the Flash Driver
        /// </summary>
        [Flags]
        private enum IOCTL
        {
            IOCTL_ERASE_FLASHFX_DISK   = 0x1,

            // New IOCTLs as of July 06
            IOCTL_STOP_FLASHFX_DISK    = 0x150011,
            IOCTL_RESUME_FLASHFX_DISK  = 0x150012,
            
            [Obsolete]IOCTL_READ_FLASH_ID        = 0x10, // Deprecated, replaced with IOCTL_READ_FLASH_SER_NUM

            // New IOCTLs as of July 06
            IOCTL_READ_FLASH_SER_NUM   = 0x150021,
            IOCTL_READ_FLASH_DEVICE_ID = 0x150022,
            IOCTL_INVALID_TEST         = 0x101010   // Invalid IOCTL for driver testing
        }
    }
}
