using System;
using OpenNETCF.IO;
using OpenNETCF.Runtime.InteropServices;

namespace AppliedData.IO
{	 
	/// <remarks>
	/// A base class that wraps the I2C Stream Interface Driver (spec 1.0) on the ADS BitsyX system
	/// </remarks>
	public abstract class I2C : StreamInterfaceDriver
	{
		/*
		#define I2C_IOCTL_RESET_BUS 					0x00
		#define I2C_IOCTL_BUS_SPEED_100KBPS				0x01
		#define I2C_IOCTL_BUS_SPEED_400KBPS				0x02
		#define I2C_IOCTL_SET_START						0x03
		#define I2C_IOCTL_SET_STOP						0x04
		#define I2C_IOCTL_READ_SCL						0x05
		#define I2C_IOCTL_READ_SDA						0x06
		#define I2C_IOCTL_READ_PXA25x_ISR				0x20
		#define I2C_IOCTL_READ_PXA25x_ICR				0x21
		#define I2C_IOCTL_GET_DRIVER_VERSION			0xA0 
		*/

		private const int RESET_BUS = 0x00;
		private const int SET_START = 0x03;
		private const int SET_STOP = 0x04;
		
		protected byte m_writeAddress;
		protected byte m_readAddress;
		private Object m_interlock = new Object();

		public I2C(byte busAddress) : base("I2C1:")
		{
			m_writeAddress = (byte)(busAddress & ~0x01);
			m_readAddress = (byte)(busAddress | 0x01);
		}
		
		protected bool Write(byte[] data, int bytesToWrite, bool setStart, bool setStop)
		{
			bool ret;

			// lock for thread safety
			lock(m_interlock)
			{
				if(setStart)
					base.DeviceIoControl(SET_START, null, null);

				ret = base.Write(data, bytesToWrite) != 0;

				if(setStop)
					base.DeviceIoControl(SET_STOP, null, null);
			}

			return ret;
		}

		protected byte[] Read(int bytesToRead, bool setStart, bool setStop)
		{
			byte[] buffer;

			// lock for thread safety
			lock(m_interlock)
			{
				if(setStart)
				{
					try
					{
						base.DeviceIoControl(SET_START, null, null);
					}
					catch(OpenNETCF.Win32.WinAPIException ex)
					{
						return null;
					}
				}
					
				try
				{
					buffer = base.Read(bytesToRead);
				}
				catch(OpenNETCF.Win32.WinAPIException ex)
				{
					return null;
				}

				if(setStop)
				{
					try
					{
						base.DeviceIoControl(SET_STOP, null, null);
					}
					catch(OpenNETCF.Win32.WinAPIException ex)
					{
						return null;
					}
				}
				
			}

			return buffer;			
		}

		public void Reset()
		{
			base.DeviceIoControl(RESET_BUS, null, null);
		}
	}

}
