using System;

namespace AppliedData.IO.Devices
{
	/// <summary>
	/// Summary description for DS1307.
	/// </summary>
	public class DS1307 : I2C
	{
		public DS1307() : base((byte)0xD0)
		{
			Open(OpenNETCF.IO.FileAccess.All, OpenNETCF.IO.FileShare.ReadWrite);
		}

		~DS1307()
		{
			Close();
		}

		public void WriteRAM(byte[] data, byte register, int bytesToWrite)
		{
			byte[] bytes = new byte[2]; // add 2 for address and register

			bytes[0]		= m_writeAddress;
			bytes[1]		= register;

			base.Write(bytes, 2, true, false);

			// send the data to the bus
			base.Write(data, bytesToWrite, false, true);
		}

		public byte[] ReadRAM(byte register, int bytesToRead)
		{
			byte[] bytes = new byte[2]; // add 2 for address and register
			byte[] data;

			bytes[0]		= m_writeAddress;
			bytes[1]		= register;

			// set address register
			Write(bytes, 2, true, true);

			bytes[0]		= m_readAddress;
			base.Write(bytes, 1, true, false);

			data = base.Read(bytesToRead, false, true);

			return data;
		}

		public DateTime ReadRTC()
		{
			byte[] header = new byte[2];
			RTCTime rtc = new RTCTime();

			// RTC data starts at zero
			// set the register pointer
			header[0]		= m_writeAddress;
			header[1]		= 0x00;
			Write(header, 2, true, true);

			// tell the device we're going to read
			// write the address with the read bit on
			header[0] = m_readAddress;
			Write(header, 1, true, false);

			// read data off the bus
			rtc = Read(rtc.Length, false, true);

			// convert the time
			return (DateTime)rtc;
		}

		public void SetRTC(DateTime newDateTime)
		{
			RTCTime rtc = new RTCTime(newDateTime);
			byte[] header = new byte[2];

			// begin the "set"
			// send the register address
			header[0]		= m_writeAddress;
			header[1]		= 0x00;

			Write(header, header.Length, true, false);

			// send the rtc data to the bus
			Write(rtc, rtc.Length, false, true);
		}

		private class RTCTime
		{
			private byte[] m_bytes = new byte[7];
			
			public RTCTime()
			{
			}

			public RTCTime(byte[] data)
			{
				m_bytes = data;
			}

			public RTCTime(DateTime dateTime)
			{
				this.Second = dateTime.Second;
				this.Minute = dateTime.Minute;
				this.Hour = dateTime.Hour;
				this.Day = dateTime.Day;
				this.Month = dateTime.Month;
				this.Year = dateTime.Year;
			}

			public static implicit operator byte[](RTCTime rtc)
			{
				return rtc.m_bytes;
			}

			public static implicit operator RTCTime(byte[] bytes)
			{
				return new RTCTime(bytes);
			}

			public static explicit operator DateTime(RTCTime rtc)
			{
				return new DateTime(rtc.Year, rtc.Month, rtc.Day, rtc.Hour, rtc.Minute, rtc.Second, 0);
			}

			public int Second
			{
				get { return BCD_TO_WORD(m_bytes[0]); }
				set { m_bytes[0] = WORD_TO_BCD(value); }
			}

			public int Minute
			{
				get { return BCD_TO_WORD(m_bytes[1]); }
				set { m_bytes[1] = WORD_TO_BCD(value); }
			}

			public int Hour
			{
				get { return BCD_TO_WORD(m_bytes[2]); }
				set { m_bytes[2] = WORD_TO_BCD(value); }
			}

			public int DayOfWeek
			{
				get { return BCD_TO_WORD(m_bytes[3]); }
				set { m_bytes[3] = WORD_TO_BCD(value); }
			}

			public int Day
			{
				get { return BCD_TO_WORD(m_bytes[4]); }
				set { m_bytes[4] = WORD_TO_BCD(value); }
			}

			public int Month
			{
				get { return BCD_TO_WORD(m_bytes[5]); }
				set { m_bytes[5] = WORD_TO_BCD(value); }
			}

			public int Year
			{
				get { return BCD_TO_WORD(m_bytes[6]); }
				set { m_bytes[6] = WORD_TO_BCD(value); }
			}

			public int Length
			{
				get { return m_bytes.Length; }
			}

			private int BCD_TO_WORD(byte x)
			{
				return (x & 0x0f) + ((x >> 4) * 10);
			}

			private byte WORD_TO_BCD(int x)
			{
				return (byte)((x % 10) + ((x / 10) * 0x10));
			}
		}	
	}
}
